@@ -47,6 +47,8 @@
 
 #include "fvm_defs.h"
 #include "fvm_config_defs.h"
+
+#include "fvm_block_dist.h"
 #include "fvm_order.h"
 #include "fvm_parall.h"
 
@@ -88,7 +90,7 @@ struct _fvm_block_to_part_t {
   size_t       n_part_ents;  /* Number of entities to receive (partition) */
   size_t       send_size;    /* Size of send buffer for MPI_Alltoall
                                 (recv_size not necessary, as recv_size
-                                should always be equal to n_recv_elts,
+                                should always be equal to n_recv_ents,
                                 though elements may arrive in a different
                                 order) */
 
@@ -328,10 +330,10 @@ _block_to_part_create(MPI_Comm comm)
  * Compute block size and rank info for use with a block distribution.
  *
  * arguments:
- *   rank_id        <-- id of local rank
+ *   rank_id        <-- id of local rank (ignored in serial mode)
  *   n_ranks        <-- number of associated ranks
  *   min_block_size <-- minimum number of entities per block
- *   n_g_elts       <-- total number of associated entities
+ *   n_g_ents       <-- total number of associated entities
  *
  * returns:
  *   block size and range info structure
@@ -341,51 +343,19 @@ fvm_block_to_part_info_t
 fvm_block_to_part_compute_sizes(int         rank_id,
                                 int         n_ranks,
                                 fvm_lnum_t  min_block_size,
-                                fvm_gnum_t  n_g_elts)
+                                fvm_gnum_t  n_g_ents)
 {
-  int _rank_id = rank_id;
-  int _n_ranks = n_ranks;
   fvm_block_to_part_info_t bi;
 
-  /* Determine rank stepping if necessary */
+  fvm_block_dist_info_t _bi= fvm_block_dist_compute_sizes(rank_id,
+                                                          n_ranks,
+                                                          min_block_size,
+                                                          n_g_ents);
 
-  bi.block_size = 0;
-  bi.rank_step = 1;
-
-  while (   n_g_elts/_n_ranks < (fvm_gnum_t)min_block_size
-         && bi.rank_step < n_ranks) {
-    bi.rank_step *= 2;
-    _n_ranks = n_ranks / bi.rank_step;
-  }
-  if (bi.rank_step > n_ranks) {
-    bi.rank_step = n_ranks;
-    _n_ranks = 1;
-  }
-
-  if (bi.rank_step > 1) {
-    if (rank_id % bi.rank_step == 0)
-      _rank_id = rank_id/bi.rank_step;
-    else
-      _rank_id = -1;
-  }
-
-  /* Now determine block size and local range */
-
-  bi.block_size = n_g_elts / _n_ranks;
-
-  if (n_g_elts % _n_ranks)
-    bi.block_size += 1;
-
-  if (_rank_id > -1) {
-    bi.gnum_range[0] = _rank_id*bi.block_size + 1;
-    bi.gnum_range[1] = (_rank_id+1)*bi.block_size + 1;
-    if (bi.gnum_range[1] > n_g_elts + 1)
-      bi.gnum_range[1] = n_g_elts + 1;
-  }
-  else {
-    bi.gnum_range[0] = 1;
-    bi.gnum_range[1] = n_g_elts + 1;
-  }
+  bi.gnum_range[0] = _bi.gnum_range[0];
+  bi.gnum_range[1] = _bi.gnum_range[1];
+  bi.rank_step = _bi.rank_step;
+  bi.block_size = _bi.block_size;
 
   return bi;
 }
@@ -500,7 +470,7 @@ fvm_block_to_part_create_by_adj_s(MPI_Comm                  comm,
   int rank = -1;
   const int n_ranks = d->n_ranks;
 
-  const fvm_lnum_t n_elts = block.gnum_range[1] - block.gnum_range[0];
+  const fvm_lnum_t n_ents = block.gnum_range[1] - block.gnum_range[0];
 
   MPI_Comm_rank(comm, &rank);
 
@@ -521,7 +491,7 @@ fvm_block_to_part_create_by_adj_s(MPI_Comm                  comm,
   for (i = 0; i < n_ranks; i++)
     adj_send_count[i] = 0;
 
-  for (j = 0; j < n_elts; j++) {
+  for (j = 0; j < n_ents; j++) {
     for (k = 0; k < stride; k++) {
       fvm_gnum_t adj_g_num = adjacency[j*stride + k];
       if (adj_g_num > 0) {
@@ -542,7 +512,7 @@ fvm_block_to_part_create_by_adj_s(MPI_Comm                  comm,
   BFT_MALLOC(adj_send_num, adj_send_size, fvm_gnum_t);
   BFT_MALLOC(adj_recv_num, adj_recv_size, fvm_gnum_t);
 
-  for (j = 0; j < n_elts; j++) {
+  for (j = 0; j < n_ents; j++) {
     for (k = 0; k < stride; k++) {
       fvm_gnum_t adj_g_num = adjacency[j*stride + k];
       if (adj_g_num > 0) {
@@ -598,7 +568,7 @@ fvm_block_to_part_create_by_adj_s(MPI_Comm                  comm,
   for (i = 0; i < n_ranks; i++)
     rank_flag[i] = -1;
 
-  for (j = 0; j < n_elts; j++) {
+  for (j = 0; j < n_ents; j++) {
     for (k = 0; k < stride; k++) {
       fvm_gnum_t adj_g_num = adjacency[j*stride + k];
       if (adj_g_num > 0) {
@@ -631,7 +601,7 @@ fvm_block_to_part_create_by_adj_s(MPI_Comm                  comm,
   for (i = 0; i < n_ranks; i++)
     rank_flag[i] = -1;
 
-  for (j = 0; j < n_elts; j++) {
+  for (j = 0; j < n_ents; j++) {
     for (k = 0; k < stride; k++) {
       fvm_gnum_t adj_g_num = adjacency[j*stride + k];
       if (adj_g_num > 0) {
@@ -895,7 +865,7 @@ fvm_block_to_part_copy_array(fvm_block_to_part_t   *d,
 
   const int n_ranks = d->n_ranks;
   const size_t _send_size = d->send_size;
-  const size_t _n_recv_elts = d->n_part_ents;
+  const size_t _n_recv_ents = d->n_part_ents;
 
   /* Adjust send and receive dimensions */
 
@@ -931,7 +901,7 @@ fvm_block_to_part_copy_array(fvm_block_to_part_t   *d,
 
   /* Distribute received values */
 
-  for (j = 0; j < _n_recv_elts; j++) {
+  for (j = 0; j < _n_recv_ents; j++) {
 
     size_t w_displ = j*stride_size;
     size_t r_displ = d->recv_order[j]*stride_size;
@@ -1022,10 +992,10 @@ fvm_block_to_part_copy_index(fvm_block_to_part_t  *d,
  *   datatype    <-- type of data considered
  *   block_index <-- local index in block distribution
  *   block_val   <-- values in block distribution
- *                   (size: send_index[n_block_elts])
+ *                   (size: send_index[n_block_ents])
  *   part_index  --> local index in general distribution
  *   part_val    --> numbers in general  distribution
- *                   (size: recv_index[n_part_elts])
+ *                   (size: recv_index[n_part_ents])
  *----------------------------------------------------------------------------*/
 
 void
