@@ -565,10 +565,9 @@ _mpi_file_close(fvm_file_t  *f)
  *   global_num_start at rank i+1 = global_num_end at rank i.
  * Otherwise, behavior (especially positioning for future reads) is undefined.
  *
- * There are 3 variants, depending on the semantics:
+ * There are 2 variants, depending on the semantics:
  *   _mpi_file_read_block_eo (using explicit offsets)
  *   _mpi_file_read_block_ip (using individual pointers, setting a file view)
- *   _mpi_file_read_block_sp (using a shared pointer and ordered semantics)
  *
  * parameters:
  *   f                <-- fvm_file_t descriptor
@@ -604,7 +603,8 @@ _mpi_file_read_block_eo(fvm_file_t  *f,
   if (errcode != MPI_SUCCESS)
     _mpi_io_error_message(f->name, errcode);
 
-  MPI_Get_count(&status, MPI_BYTE, &count);
+  if (count > 0)
+    MPI_Get_count(&status, MPI_BYTE, &count);
   retval = count / size;
 
   MPI_Bcast(&global_num_end_last, 1, FVM_MPI_GNUM, f->n_ranks-1, f->comm);
@@ -620,16 +620,19 @@ _mpi_file_read_block_ip(fvm_file_t  *f,
                         fvm_gnum_t   global_num_start,
                         fvm_gnum_t   global_num_end)
 {
+  int errcode;
+  int lengths[1];
+  MPI_Aint disps[1];
+  MPI_Status status;
+  MPI_Datatype file_type;
+
+  int count = 0;
   char datarep[] = "native";
   fvm_gnum_t global_num_end_last = global_num_end;
   size_t retval = 0;
 
-  MPI_Status status;
-  MPI_Datatype file_type;
-  int errcode, count;
-
-  int lengths[1] = {(global_num_end - global_num_start) * size};
-  MPI_Aint disps[1] = {(global_num_start - 1) * size};
+  lengths[0] = (global_num_end - global_num_start) * size;
+  disps[0] = (global_num_start - 1) * size;
 
   MPI_Type_hindexed(1, lengths, disps, MPI_BYTE, &file_type);
   MPI_Type_commit(&file_type);
@@ -644,38 +647,8 @@ _mpi_file_read_block_ip(fvm_file_t  *f,
 
   MPI_Type_free(&file_type);
 
-  MPI_Get_count(&status, MPI_BYTE, &count);
-  retval = count / size;
-
-  MPI_Bcast(&global_num_end_last, 1, FVM_MPI_GNUM, f->n_ranks-1, f->comm);
-  f->offset += ((global_num_end_last - 1) * size);
-
-  return retval;
-}
-
-static size_t
-_mpi_file_read_block_sp(fvm_file_t  *f,
-                        void        *buf,
-                        size_t       size,
-                        fvm_gnum_t   global_num_start,
-                        fvm_gnum_t   global_num_end)
-{
-  fvm_gnum_t global_num_end_last = global_num_end;
-  size_t retval = 0;
-
-  MPI_Offset disp;
-  MPI_Status status;
-  int errcode, count;
-
-  disp = f->offset + ((global_num_start - 1) * size);
-  count = (global_num_end - global_num_start)*size;
-
-  errcode = MPI_File_read_ordered(f->fh, buf, count, MPI_BYTE, &status);
-
-  if (errcode != MPI_SUCCESS)
-    _mpi_io_error_message(f->name, errcode);
-
-  MPI_Get_count(&status, MPI_BYTE, &count);
+  if (lengths[0] > 0)
+    MPI_Get_count(&status, MPI_BYTE, &count);
   retval = count / size;
 
   MPI_Bcast(&global_num_end_last, 1, FVM_MPI_GNUM, f->n_ranks-1, f->comm);
@@ -694,10 +667,9 @@ _mpi_file_read_block_sp(fvm_file_t  *f,
  *   global_num_start at rank i+1 = global_num_end at rank i.
  * Otherwise, behavior (especially positioning for future reads) is undefined.
  *
- * There are 3 variants, depending on the semantics:
+ * There are 2 variants, depending on the semantics:
  *   _mpi_file_write_block_eo (using explicit offsets)
  *   _mpi_file_write_block_ip (using individual pointers, setting a file view)
- *   _mpi_file_write_block_sp (using a shared pointer and ordered semantics)
  *
  * parameters:
  *   f                <-- fvm_file_t descriptor
@@ -733,7 +705,8 @@ _mpi_file_write_block_eo(fvm_file_t  *f,
   if (errcode != MPI_SUCCESS)
     _mpi_io_error_message(f->name, errcode);
 
-  MPI_Get_count(&status, MPI_BYTE, &count);
+  if (count > 0)
+    MPI_Get_count(&status, MPI_BYTE, &count);
   retval = count / size;
 
   MPI_Bcast(&global_num_end_last, 1, FVM_MPI_GNUM, f->n_ranks-1, f->comm);
@@ -749,16 +722,18 @@ _mpi_file_write_block_ip(fvm_file_t  *f,
                          fvm_gnum_t   global_num_start,
                          fvm_gnum_t   global_num_end)
 {
+  int lengths[1];
+  MPI_Aint disps[1];
+  MPI_Status status;
+  MPI_Datatype file_type;
+
+  int errcode = MPI_SUCCESS, count = 0;
   char datarep[] = "native";
   fvm_gnum_t global_num_end_last = global_num_end;
   size_t retval = 0;
 
-  MPI_Status status;
-  MPI_Datatype file_type;
-  int errcode = MPI_SUCCESS, count = 0;
-
-  int lengths[1] = {(global_num_end - global_num_start) * size};
-  MPI_Aint disps[1] = {(global_num_start - 1) * size};
+  lengths[0] = (global_num_end - global_num_start) * size;
+  disps[0] = (global_num_start - 1) * size;
 
   MPI_Type_hindexed(1, lengths, disps, MPI_BYTE, &file_type);
   MPI_Type_commit(&file_type);
@@ -783,37 +758,6 @@ _mpi_file_write_block_ip(fvm_file_t  *f,
   return retval;
 }
 
-static size_t
-_mpi_file_write_block_sp(fvm_file_t  *f,
-                         void        *buf,
-                         size_t       size,
-                         fvm_gnum_t   global_num_start,
-                         fvm_gnum_t   global_num_end)
-{
-  fvm_gnum_t global_num_end_last = global_num_end;
-  size_t retval = 0;
-
-  MPI_Offset disp;
-  MPI_Status status;
-  int errcode, count;
-
-  disp = f->offset + ((global_num_start - 1) * size);
-  count = (global_num_end - global_num_start)*size;
-
-  errcode = MPI_File_write_ordered(f->fh, buf, count, MPI_BYTE, &status);
-
-  if (errcode != MPI_SUCCESS)
-    _mpi_io_error_message(f->name, errcode);
-
-  MPI_Get_count(&status, MPI_BYTE, &count);
-  retval = count / size;
-
-  MPI_Bcast(&global_num_end_last, 1, FVM_MPI_GNUM, f->n_ranks-1, f->comm);
-  f->offset += ((global_num_end_last - 1) * size);
-
-  return retval;
-}
-
 #endif /* defined(FVM_HAVE_MPI_IO) */
 
 /*=============================================================================
@@ -902,8 +846,7 @@ fvm_file_open(const char         *name,
   if (   f->comm != MPI_COMM_NULL
       && !(hints & FVM_FILE_NO_MPI_IO)) {
     int positioning_mask = (  FVM_FILE_EXPLICIT_OFFSETS
-                            | FVM_FILE_INDIVIDUAL_POINTERS
-                            | FVM_FILE_SHARED_POINTER);
+                            | FVM_FILE_INDIVIDUAL_POINTERS);
     if (hints & positioning_mask)
       f->semantics = hints & positioning_mask;
     else
@@ -1110,10 +1053,12 @@ fvm_file_read_global(fvm_file_t  *f,
       }
 
       else if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS) {
-        char datarep[] = "native";
-        int lengths[1] = {ni * size};
         MPI_Datatype file_type;
-        MPI_Aint disps[1] = {0};
+        MPI_Aint disps[1];
+        int lengths[1];
+        char datarep[] = "native";
+        lengths[0] = ni * size;
+        disps[0] = 0;
         MPI_Type_hindexed(1, lengths, disps, MPI_BYTE, &file_type);
         MPI_Type_commit(&file_type);
         MPI_File_set_view(f->fh, f->offset, MPI_BYTE, file_type,
@@ -1127,22 +1072,6 @@ fvm_file_read_global(fvm_file_t  *f,
         MPI_Type_free(&file_type);
       }
 
-      else { /* if (f->semantics & FVM_FILE_SHARED_POINTER) */
-        int aux[2];
-        if (f->rank == 0) {
-          aux[0] = MPI_File_read_shared(f->fh,
-                                        buf,
-                                        size*ni,
-                                        MPI_BYTE,
-                                        &status);
-          MPI_Get_count(&status, MPI_BYTE, &aux[1]);
-        }
-        MPI_Bcast(buf, size*ni, MPI_BYTE, 0, f->comm);
-        MPI_Bcast(aux, 2, MPI_INT, 0, f->comm);
-        errcode = aux[0];
-        count = aux[1];
-      }
-
       if (errcode != MPI_SUCCESS)
         _mpi_io_error_message(f->name, errcode);
 
@@ -1175,7 +1104,7 @@ fvm_file_read_global(fvm_file_t  *f,
  *   ni   <-- number of items to read
  *
  * returns:
- *   the number of items (not bytes) sucessfully read; currently,
+ *   the number of items (not bytes) sucessfully written; currently,
  *   errors are fatal.
  *----------------------------------------------------------------------------*/
 
@@ -1185,7 +1114,7 @@ fvm_file_write_global(fvm_file_t  *f,
                       size_t       size,
                       size_t       ni)
 {
-  size_t retval = 0;
+  size_t retval = ni;
 
   unsigned char _copybuf[1024];
   unsigned char *copybuf = _copybuf;
@@ -1234,10 +1163,12 @@ fvm_file_write_global(fvm_file_t  *f,
     }
 
     else if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS) {
-      char datarep[] = "native";
-      int lengths[1] = {ni * size};
       MPI_Datatype file_type;
-      MPI_Aint disps[1] = {0};
+      MPI_Aint disps[1];
+      int lengths[1];
+      char datarep[] = "native";
+      lengths[0] = ni * size;
+      disps[0] = 0;
       MPI_Type_hindexed(1, lengths, disps, MPI_BYTE, &file_type);
       MPI_Type_commit(&file_type);
       MPI_File_set_view(f->fh, f->offset, MPI_BYTE, file_type,
@@ -1253,17 +1184,6 @@ fvm_file_write_global(fvm_file_t  *f,
       MPI_Type_free(&file_type);
     }
 
-    else { /* if (f->semantics & FVM_FILE_SHARED_POINTER) */
-      if (f->rank == 0) {
-        aux[0] = MPI_File_write_shared(f->fh,
-                                       copybuf,
-                                       size*ni,
-                                       MPI_BYTE,
-                                       &status);
-        MPI_Get_count(&status, MPI_BYTE, &(aux[1]));
-      }
-    }
-
     MPI_Bcast(aux, 2, MPI_INT, 0, f->comm);
 
     if (aux[0] != MPI_SUCCESS)
@@ -1337,19 +1257,12 @@ fvm_file_read_block(fvm_file_t  *f,
                                        _global_num_start,
                                        _global_num_end);
 
-    else if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS)
+    else /* if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS) */
       retval = _mpi_file_read_block_ip(f,
                                        buf,
                                        size,
                                        _global_num_start,
                                        _global_num_end);
-
-    else /* if (f->semantics & FVM_FILE_SHARED_POINTER) */
-      retval = _mpi_file_read_block_sp(f,
-                                       buf,
-                                       size,
-                                       _global_num_start,
-                                       _global_num_end);
   }
 
 #endif /* defined(FVM_HAVE_MPI_IO) */
@@ -1511,19 +1424,12 @@ fvm_file_write_block_buffer(fvm_file_t  *f,
                                         _global_num_start,
                                         _global_num_end);
 
-    else if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS)
+    else /* if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS) */
       retval = _mpi_file_write_block_ip(f,
                                         buf,
                                         size,
                                         _global_num_start,
                                         _global_num_end);
-
-    else /* if (f->semantics & FVM_FILE_SHARED_POINTER) */
-      retval = _mpi_file_write_block_sp(f,
-                                        buf,
-                                        size,
-                                        _global_num_start,
-                                        _global_num_end);
   }
 
 #endif /* defined(FVM_HAVE_MPI_IO) */
@@ -1585,11 +1491,6 @@ fvm_file_seek(fvm_file_t       *f,
                              (MPI_Offset)offset,
                              _mpi_seek[whence]);
 
-    else /* if (f->semantics & FVM_FILE_SHARED_POINTER) */
-      retval = MPI_File_seek_shared(f->fh,
-                                    (MPI_Offset)offset,
-                                    _mpi_seek[whence]);
-
     if (retval != MPI_SUCCESS)
       _mpi_io_error_message(f->name, retval);
   }
@@ -1637,16 +1538,16 @@ fvm_file_tell(fvm_file_t  *f)
   else if (!(f->semantics & FVM_FILE_NO_MPI_IO)) {
 
     int errcode = MPI_SUCCESS;
-    MPI_Offset offset, disp;
 
     if (f->semantics & FVM_FILE_EXPLICIT_OFFSETS)
       retval = f->offset;
 
-    else if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS) {
+    else { /* if (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS) */
 #if 0
       /* This should work, but fails after certain collective writes
          with some processes writing zero values (at least on
          Open MPI 1.2.6, using ROMIO). */
+      MPI_Offset offset, disp;
       int64_t l_offset, g_offset;
       MPI_Datatype offset_type = fvm_datatype_to_mpi[FVM_INT64];
       errcode = MPI_File_get_position(f->fh, &offset);
@@ -1664,35 +1565,6 @@ fvm_file_tell(fvm_file_t  *f)
 #endif
     }
 
-    else { /* if (f->semantics & FVM_FILE_SHARED_POINTER) */
-
-#if 0
-      /* It seems this should work according to the MPI 2 standard,
-         but it may hang on both OpenMPI 1.2.6 and MPICH2 1.0.7
-         (both use ROMIO) on NFS file systems */
-      errcode = MPI_File_get_position_shared(f->fh, &offset);
-      if (errcode == MPI_SUCCESS) {
-        MPI_File_get_byte_offset(f->fh, offset, &disp);
-        retval = disp;
-      }
-#else /* Work around apparent bug using ROMIO on NFS (NFS locking bug) */
-      int64_t aux[2];
-      MPI_Datatype offset_type = fvm_datatype_to_mpi[FVM_INT64];
-      if (f->rank == 0) {
-        errcode = MPI_File_get_position_shared(f->fh, &offset);
-        if (errcode == MPI_SUCCESS) {
-          MPI_File_get_byte_offset(f->fh, offset, &disp);
-          retval = disp;
-        }
-        aux[0] = disp; aux[1] = retval;
-      }
-      MPI_Bcast(aux, 2, offset_type, 0, f->comm);
-      disp = aux[0];
-      retval = aux[1];
-
-#endif
-    }
-
     if (errcode != MPI_SUCCESS)
       _mpi_io_error_message(f->name, errcode);
   }
@@ -1730,7 +1602,6 @@ fvm_file_dump(const fvm_file_t  *f)
                "  no_predistribute:       %d\n"
                "  explicit_offsets:       %d\n"
                "  individual_pointers:    %d\n"
-               "  shared_pointer:         %d\n"
                "Rank:                     %d\n"
                "N ranks:                  %d\n"
                "Swap endian:              %d\n"
@@ -1740,7 +1611,6 @@ fvm_file_dump(const fvm_file_t  *f)
              (f->semantics & FVM_FILE_NO_PREDISTRIBUTE) >> 1,
              (f->semantics & FVM_FILE_EXPLICIT_OFFSETS) >> 2,
              (f->semantics & FVM_FILE_INDIVIDUAL_POINTERS) >> 3,
-             (f->semantics & FVM_FILE_SHARED_POINTER) >> 4,
              f->rank, f->n_ranks, (int)(f->swap_endian),
              f->sh);
 
