@@ -243,7 +243,7 @@ _fvm_io_num_global_order(fvm_io_num_t       *this_io_num,
 
   _Bool       may_be_shared = false;
 
-  fvm_gnum_t  *global_num_shift = NULL, *recv_global_num = NULL;
+  fvm_gnum_t  *recv_global_num = NULL;
   fvm_lnum_t  *recv_n_sub = NULL, *recv_order = NULL;
   int         *send_count = NULL, *recv_count = NULL;
   int         *send_shift = NULL, *recv_shift = NULL;
@@ -251,7 +251,7 @@ _fvm_io_num_global_order(fvm_io_num_t       *this_io_num,
 
   int         local_rank, size;
 
-  fvm_gnum_t  current_global_num = 0;
+  fvm_gnum_t  current_global_num = 0, global_num_shift = 0;
 
   /* Initialization */
 
@@ -403,21 +403,12 @@ _fvm_io_num_global_order(fvm_io_num_t       *this_io_num,
      processes by the cumulative total number of entities handled by
      each process */
 
-  BFT_MALLOC(global_num_shift, size, fvm_gnum_t);
+  MPI_Scan(&current_global_num, &global_num_shift, 1, FVM_MPI_GNUM,
+           MPI_SUM, comm);
+  global_num_shift -= current_global_num;
 
-  MPI_Allgather(&current_global_num, 1, FVM_MPI_GNUM,
-                global_num_shift, 1, FVM_MPI_GNUM, comm);
-
-  for (rank = 1; rank < size; rank++)
-    global_num_shift[rank] =   global_num_shift[rank-1]
-                             + global_num_shift[rank];
-
-  if (local_rank > 0) {
-    for (i = 0; i < n_ent_recv; i++)
-      recv_global_num[i] += global_num_shift[local_rank - 1];
-  }
-
-  BFT_FREE(global_num_shift);
+  for (i = 0; i < n_ent_recv; i++)
+    recv_global_num[i] += global_num_shift;
 
   /* Return global order to all processors */
 
@@ -482,6 +473,192 @@ _fvm_io_num_global_order(fvm_io_num_t       *this_io_num,
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Global ordering associated with an I/O numbering structure.
+ *
+ * The structure should contain an initial ordering, which should
+ * be sorted, but need not be contiguous. On output, the numbering
+ * will be contiguous.
+ *
+ * As an option, a number of sub-entities per initial entity may be
+ * given, in which case sub-entities of a same entity will have contiguous
+ * numbers in the final ordering.
+ *
+ * parameters:
+ *   this_io_num    <-> pointer to structure that should be ordered
+ *   n_sub_entities <-- optional number of sub-entities per initial entity,
+ *                      or NULL if unused
+ *   comm           <-- associated MPI communicator
+ *----------------------------------------------------------------------------*/
+
+static void
+_fvm_io_num_global_order_s(fvm_io_num_t       *this_io_num,
+                           size_t              stride,
+                           fvm_gnum_t          global_num[],
+                           MPI_Comm            comm)
+{
+  fvm_gnum_t  n_ent_recv;
+  size_t      i, j, slice_size;
+  int         rank;
+
+  fvm_gnum_t  *block_global_num = NULL, *recv_global_num = NULL;
+  fvm_lnum_t  *recv_n_sub = NULL, *recv_order = NULL;
+  int         *send_count = NULL, *recv_count = NULL;
+  int         *send_shift = NULL, *recv_shift = NULL;
+
+  int         local_rank, size;
+
+  fvm_gnum_t  current_global_num = 0, global_num_shift = 0;
+
+  /* Initialization */
+
+  MPI_Comm_rank(comm, &local_rank);
+  MPI_Comm_size(comm, &size);
+
+  /* Get maximum global number value for first value of each series
+     (does not need to be exact, simply used to define blocks) */
+
+  {
+    fvm_gnum_t  local_max = 0, global_max = 0;
+    size_t      n_ent = this_io_num->global_num_size;
+
+    if (n_ent > 0)
+      local_max = global_num[(n_ent-1)*2];
+    MPI_Allreduce(&local_max, &global_max, 1, FVM_MPI_GNUM, MPI_MAX, comm);
+    this_io_num->global_count = global_max;
+  }
+
+  /* slice_size = ceil(this_io_num->global_count/size) */
+
+  slice_size = this_io_num->global_count / size;
+  if (this_io_num->global_count % size > 0)
+    slice_size += 1;
+
+  assert(sizeof(fvm_gnum_t) >= sizeof(fvm_lnum_t));
+
+  BFT_MALLOC(send_count, size, int);
+  BFT_MALLOC(recv_count, size, int);
+
+  BFT_MALLOC(send_shift, size, int);
+  BFT_MALLOC(recv_shift, size, int);
+
+  /* Count number of values to send to each process */
+
+  for (rank = 0; rank < size; rank++)
+    send_count[rank] = 0;
+
+  for (i = 0; i < (size_t)(this_io_num->global_num_size); i++)
+    send_count[(global_num[stride*i] - 1) / slice_size] += stride;
+
+  MPI_Alltoall(send_count, 1, FVM_MPI_GNUM, recv_count, 1, FVM_MPI_GNUM, comm);
+
+  send_shift[0] = 0;
+  recv_shift[0] = 0;
+
+  for (rank = 1; rank < size; rank++) {
+    send_shift[rank] = send_shift[rank - 1] + send_count[rank -1];
+    recv_shift[rank] = recv_shift[rank - 1] + recv_count[rank -1];
+  }
+
+  /* As data is sorted by increasing base global numbering, we do not
+     need to build an extra array, but only to send the correct parts
+     of the n_sub_entities[] array to the correct processors */
+
+  n_ent_recv = (recv_shift[size - 1] + recv_count[size - 1]) / stride;
+
+  BFT_MALLOC(recv_global_num, stride*n_ent_recv, fvm_gnum_t);
+  BFT_MALLOC(recv_order, n_ent_recv, fvm_lnum_t);
+
+  MPI_Alltoallv(global_num, send_count, send_shift, FVM_MPI_GNUM,
+                recv_global_num, recv_count, recv_shift, FVM_MPI_GNUM, comm);
+
+  if (n_ent_recv > 0) {
+
+    size_t prev_id, cur_id;
+
+    fvm_order_local_allocated_s(NULL,
+                                recv_global_num,
+                                stride,
+                                recv_order,
+                                n_ent_recv);
+
+    /* Determine global order; requires ordering to loop through buffer by
+       increasing number (slice blocks associated with each process are
+       already sorted, but the whole "gathered" slice is not).
+       We build an initial global order based on the initial global numbering,
+       such that for each slice, the global number of an entity is equal to
+       the cumulative number of sub-entities */
+
+    BFT_MALLOC(block_global_num, n_ent_recv, fvm_gnum_t);
+
+    current_global_num = 1;
+    prev_id = recv_order[0];
+    block_global_num[recv_order[0]] = current_global_num;
+
+    for (i = 1; i < n_ent_recv; i++) {
+      bool greater_than_prev = false;
+      cur_id = recv_order[i];
+      for (j = 0; j < stride; j++) {
+        if (  recv_global_num[cur_id*stride + j]
+            > recv_global_num[prev_id*stride + j])
+          greater_than_prev = true;
+      }
+      if (greater_than_prev)
+        current_global_num += 1;
+      block_global_num[recv_order[i]] = current_global_num;
+      prev_id = cur_id;
+    }
+
+  }
+
+  /* Partial clean-up */
+
+  BFT_FREE(recv_n_sub);
+  BFT_FREE(recv_order);
+  BFT_FREE(recv_global_num);
+
+  /* At this stage, recv_global_num[] is valid for this process, and
+     current_global_num indicates the total number of entities handled
+     by this process; we must now shift global numberings on different
+     processes by the cumulative total number of entities handled by
+     each process */
+
+  MPI_Scan(&current_global_num, &global_num_shift, 1, FVM_MPI_GNUM,
+           MPI_SUM, comm);
+  global_num_shift -= current_global_num;
+
+  for (i = 0; i < n_ent_recv; i++)
+    block_global_num[i] += global_num_shift;
+
+  /* Return global order to all processors */
+
+  for (rank = 0; rank < size; rank++) {
+    send_count[rank] /= stride;
+    recv_count[rank] /= stride;
+  }
+
+  for (rank = 1; rank < size; rank++) {
+    send_shift[rank] = send_shift[rank - 1] + send_count[rank -1];
+    recv_shift[rank] = recv_shift[rank - 1] + recv_count[rank -1];
+  }
+
+  MPI_Alltoallv(block_global_num, recv_count, recv_shift, FVM_MPI_GNUM,
+                this_io_num->_global_num, send_count, send_shift, FVM_MPI_GNUM,
+                comm);
+
+  /* Free memory */
+
+  BFT_FREE(block_global_num);
+  BFT_FREE(send_count);
+  BFT_FREE(recv_count);
+  BFT_FREE(send_shift);
+  BFT_FREE(recv_shift);
+
+  /* Get final maximum global number value */
+
+  this_io_num->global_count = _fvm_io_num_global_max(this_io_num, comm);
+}
+
 /*----------------------------------------------------------------------------
  * Return the global number of sub-entities associated with an initial
  * entity whose global numbering is known, given the number of
@@ -691,10 +868,10 @@ _fvm_io_num_global_sub_size(const fvm_io_num_t  *this_io_num,
 fvm_io_num_t *
 fvm_io_num_create(const fvm_lnum_t  parent_entity_number[],
                   const fvm_gnum_t  parent_global_number[],
-                  const size_t      n_entities,
-                  const int         share_parent_global)
+                  size_t            n_entities,
+                  int               share_parent_global)
 {
-  fvm_io_num_t  *this_io_num;
+  fvm_io_num_t  *this_io_num = NULL;
 
   /* Initial checks */
 
@@ -771,7 +948,7 @@ fvm_io_num_create(const fvm_lnum_t  parent_entity_number[],
 fvm_io_num_t *
 fvm_io_num_create_shared(const fvm_gnum_t  global_number[],
                          fvm_gnum_t        global_count,
-                         const size_t      n_entities)
+                         size_t            n_entities)
 {
   fvm_io_num_t  *this_io_num;
 
@@ -850,6 +1027,90 @@ fvm_io_num_create_from_sub(const fvm_io_num_t  *base_io_num,
   return this_io_num;
 }
 
+/*----------------------------------------------------------------------------
+ * Creation of an I/O numbering structure based on a strided adjacency.
+ *
+ * The corresponding entities must be locally ordered.
+ *
+ * parameters:
+ *   parent_entity_number <-- pointer to list of selected entitie's parent's
+ *                            numbers, or NULL if all first n_ent entities
+ *                            are used
+ *   adjacency            <-- entity adjacency (1 to n global numbering)
+ *   n_entities           <-- number of entities considered
+ *   stride               <-- values per entity
+ *
+ * returns:
+ *  pointer to I/O numbering structure
+ *----------------------------------------------------------------------------*/
+
+fvm_io_num_t *
+fvm_io_num_create_from_adj_s(const fvm_lnum_t  parent_entity_number[],
+                             const fvm_gnum_t  adjacency[],
+                             size_t            n_entities,
+                             size_t            stride)
+{
+  fvm_gnum_t *_adjacency = NULL;
+
+  fvm_io_num_t  *this_io_num = NULL;
+
+  /* Initial checks */
+
+  if (fvm_parall_get_size() < 2)
+    return NULL;
+
+  assert(fvm_order_local_test_s(parent_entity_number,
+                                adjacency,
+                                stride,
+                                n_entities) == true);
+
+#if defined(FVM_HAVE_MPI)
+
+  /* Create structure */
+
+  BFT_MALLOC(this_io_num, 1, fvm_io_num_t);
+
+  this_io_num->global_num_size = n_entities;
+
+  BFT_MALLOC(this_io_num->_global_num, n_entities, fvm_gnum_t);
+  this_io_num->global_num = this_io_num->_global_num;
+
+  if (n_entities > 0) {
+
+    size_t  i, j;
+
+    /* Assign initial global numbers */
+
+    BFT_MALLOC(_adjacency, n_entities*stride, fvm_gnum_t);
+
+    if (parent_entity_number != NULL) {
+      for (i = 0 ; i < n_entities ; i++) {
+        for (j = 0; j < stride; j++)
+          _adjacency[i*stride + j]
+            = adjacency[(parent_entity_number[i]-1)*stride + j];
+      }
+    }
+    else
+      memcpy(_adjacency, adjacency, n_entities*stride*sizeof(fvm_gnum_t));
+
+  }
+
+  /* Order globally */
+
+  this_io_num->global_count = n_entities;
+
+  _fvm_io_num_global_order_s(this_io_num,
+                             stride,
+                             _adjacency,
+                             fvm_parall_get_mpi_comm());
+
+  BFT_FREE(_adjacency);
+
+#endif
+
+  return this_io_num;
+}
+
 /*----------------------------------------------------------------------------
  * Destruction of a I/O numbering structure.
  *
