@@ -1281,13 +1281,12 @@ _exchange_point_var_distant(fvm_locator_t     *this_locator,
 
     _locator_trace_end_comm(_fvm_locator_log_end_p_comm, comm_timing);
 
-    if (  (loc_v_flag == 1 && (local_var == NULL || n_points_loc == 0))
-        || (loc_v_flag == 0 && n_points_loc > 0))
-        bft_error(__FILE__, __LINE__, 0,
-                  _("Incoherent arguments to different instances in "
-                    "_exchange_point_var().\n"
-                    "Send and receive operations do not match "
-                    "(dist_rank = %d\n)\n"), dist_rank);
+    if (loc_v_flag == 1 && (local_var == NULL || n_points_loc == 0))
+      bft_error(__FILE__, __LINE__, 0,
+                _("Incoherent arguments to different instances in "
+                  "_exchange_point_var().\n"
+                  "Send and receive operations do not match "
+                  "(dist_rank = %d\n)\n"), dist_rank);
 
     dist_v_idx = this_locator->distant_points_idx[i] * stride*size;
     dist_v_count = n_points_dist * stride * dist_v_flag;
@@ -1314,56 +1313,60 @@ _exchange_point_var_distant(fvm_locator_t     *this_locator,
 
       _locator_trace_end_comm(_fvm_locator_log_end_p_comm, comm_timing);
 
-      if (local_list == NULL) {
-        int k;
-        size_t l;
-        const size_t nbytes = stride*size;
-        for (k = 0; k < n_points_loc; k++) {
-          char *local_v_p = (char *)local_var + _local_point_ids[k]*nbytes;
-          const char *loc_v_buf_p = (const char *)loc_v_buf + k*nbytes;
-          for (l = 0; l < nbytes; l++)
-            local_v_p[l] = loc_v_buf_p[l];
+      if (loc_v_flag > 0) {
+        if (local_list == NULL) {
+          int k;
+          size_t l;
+          const size_t nbytes = stride*size;
+          for (k = 0; k < n_points_loc; k++) {
+            char *local_v_p = (char *)local_var + _local_point_ids[k]*nbytes;
+            const char *loc_v_buf_p = (const char *)loc_v_buf + k*nbytes;
+            for (l = 0; l < nbytes; l++)
+              local_v_p[l] = loc_v_buf_p[l];
+          }
         }
-      }
-      else {
-        int k;
-        size_t l;
-        const size_t nbytes = stride*size;
-        for (k = 0; k < n_points_loc; k++) {
-          char *local_v_p =   (char *)local_var
-                            + (local_list[_local_point_ids[k]] - 1)*nbytes;
-          const char *loc_v_buf_p = (const char *)loc_v_buf + k*nbytes;
-          for (l = 0; l < nbytes; l++)
-            local_v_p[l] = loc_v_buf_p[l];
+        else {
+          int k;
+          size_t l;
+          const size_t nbytes = stride*size;
+          for (k = 0; k < n_points_loc; k++) {
+            char *local_v_p =   (char *)local_var
+                              + (local_list[_local_point_ids[k]] - 1)*nbytes;
+            const char *loc_v_buf_p = (const char *)loc_v_buf + k*nbytes;
+            for (l = 0; l < nbytes; l++)
+              local_v_p[l] = loc_v_buf_p[l];
+          }
         }
       }
 
     }
     else { /* if (reverse == true) */
 
-      if (local_list == NULL) {
-        int k;
-        size_t l;
-        const size_t nbytes = stride*size;
-        for (k = 0; k < n_points_loc; k++) {
-          const char *local_v_p
-            = (const char *)local_var + _local_point_ids[k]*nbytes;
-          char *loc_v_buf_p = (char *)loc_v_buf + k*nbytes;
-          for (l = 0; l < nbytes; l++)
-            loc_v_buf_p[l] = local_v_p[l];
+      if (loc_v_flag > 0) {
+        if (local_list == NULL) {
+          int k;
+          size_t l;
+          const size_t nbytes = stride*size;
+          for (k = 0; k < n_points_loc; k++) {
+            const char *local_v_p
+              = (const char *)local_var + _local_point_ids[k]*nbytes;
+            char *loc_v_buf_p = (char *)loc_v_buf + k*nbytes;
+            for (l = 0; l < nbytes; l++)
+              loc_v_buf_p[l] = local_v_p[l];
+          }
         }
-      }
-      else {
-        int k;
-        size_t l;
-        const size_t nbytes = stride*size;
-        for (k = 0; k < n_points_loc; k++) {
-          const char *local_v_p
-            = (const char *)local_var
+        else {
+          int k;
+          size_t l;
+          const size_t nbytes = stride*size;
+          for (k = 0; k < n_points_loc; k++) {
+            const char *local_v_p
+              = (const char *)local_var
                 + (local_list[_local_point_ids[k]] - 1)*nbytes;
-          char *loc_v_buf_p = (char *)loc_v_buf + k*nbytes;
-          for (l = 0; l < nbytes; l++)
-            loc_v_buf_p[l] = local_v_p[l];
+            char *loc_v_buf_p = (char *)loc_v_buf + k*nbytes;
+            for (l = 0; l < nbytes; l++)
+              loc_v_buf_p[l] = local_v_p[l];
+          }
         }
       }
 
