@@ -338,6 +338,89 @@ fvm_order_local_test(const fvm_lnum_t  list[],
     return 0;
 }
 
+/*----------------------------------------------------------------------------
+ * Test if an array of global numbers is lexicographically ordered.
+ *
+ * parameters:
+ *   list     <-- optional list (1 to n numbering) of selected entities
+ *                (or NULL if all nb_ent are selected). This list may
+ *                contain element numbers in any order
+ *   number   <-- array of all entity numbers (number of entity i
+ *                given by number[i] or number[list[i] - 1]) if list exists
+ *                (if NULL, a default 1 to n numbering is considered)
+ *   stride   <-- stride of number array (number of values to compare)
+ *   nb_ent   <-- number of entities considered
+ *
+ * returns:
+ *   1 if ordered, 0 otherwise.
+ *----------------------------------------------------------------------------*/
+
+int
+fvm_order_local_test_s(const fvm_lnum_t  list[],
+                       const fvm_gnum_t  number[],
+                       size_t            stride,
+                       size_t            nb_ent)
+{
+  size_t j;
+  size_t i = 0;
+
+  /* If numbering is explicit */
+
+  if (number != NULL) {
+
+    if (list != NULL) {
+      for (i = 1 ; i < nb_ent ; i++) {
+        size_t j_prev, k;
+        bool unordered = false;
+        j_prev = list[i-1] - 1;
+        j = list[i] - 1;
+        for (k = 0; k < stride; k++) {
+          if (number[j_prev*stride + k] < number[j*stride + k])
+            break;
+          else if (number[j_prev*stride + k] > number[j*stride + k])
+            unordered = true;
+        }
+        if (unordered == true)
+          break;
+      }
+    }
+    else {
+      for (i = 1 ; i < nb_ent ; i++) {
+        size_t i_prev, k;
+        bool unordered = false;
+        i_prev = i-1;
+        for (k = 0; k < stride; k++) {
+          if (number[i_prev*stride + k] < number[i*stride + k])
+            break;
+          else if (number[i_prev*stride + k] > number[i*stride + k])
+            unordered = true;
+        }
+        if (unordered == true)
+          break;
+      }
+    }
+
+  /* If numbering is implicit */
+
+  }
+  else {
+
+    if (list != NULL) {
+      for (i = 1 ; i < nb_ent ; i++) {
+        if (list[i] < list[i-1])
+          break;
+      }
+    }
+    else
+      i = nb_ent;
+  }
+
+  if (i == nb_ent || nb_ent == 0)
+    return 1;
+  else
+    return 0;
+}
+
 /*----------------------------------------------------------------------------
  * Return an ordering table associated with an array of global numbers.
  *
