@@ -212,13 +212,15 @@ _combine_transforms_check(const fvm_periodicity_t  *this_periodicity,
                           _Bool                     abort_on_error)
 {
   int i;
+  int eq[3];
+  int rev[3] = {-1, -1, -1};
   const _transform_t  *tr0 = this_periodicity->transform[tr_id_0];
   const _transform_t  *tr1 = this_periodicity->transform[tr_id_1];
-  int eq[3] = {tr_id_0, tr_id_1, -1};
-  int rev[3] = {-1, -1, -1};
 
   assert(tr0->parent_ids[0] < 0 && tr0->parent_ids[1] < 0);
 
+  eq[0] = tr_id_0; eq[1] = tr_id_1; eq[2] = -1;
+
   /* Check that all components are independant */
 
   if (tr1->parent_ids[1] > -1) {
@@ -788,9 +790,13 @@ fvm_periodicity_add_translation(fvm_periodicity_t  *this_periodicity,
                                 int                 external_num,
                                 const double        translation[3])
 {
-  double  matrix[3][4] = {{1., 0., 0., translation[0]},
-                          {0., 1., 0., translation[1]},
-                          {0., 0., 1., translation[2]}};
+  double  matrix[3][4] = {{1., 0., 0., 0.},
+                          {0., 1., 0., 0.},
+                          {0., 0., 1., 0.}};
+
+  matrix[0][3] = translation[0];
+  matrix[1][3] = translation[1];
+  matrix[2][3] = translation[2];
 
   return fvm_periodicity_add_by_matrix(this_periodicity,
                                        external_num,
@@ -1094,8 +1100,9 @@ fvm_periodicity_get_components(const fvm_periodicity_t  *this_periodicity,
       component_ids[2] = -1;
     }
     else {
+      _transform_t  *tr_1;
       assert(tr_id_1 < this_periodicity->tr_level_idx[2]);
-      _transform_t  *tr_1 = this_periodicity->transform[tr_id_1];
+      tr_1 = this_periodicity->transform[tr_id_1];
       component_ids[1] = tr_1->parent_ids[0];
       component_ids[2] = tr_1->parent_ids[1];
     }
