@@ -2478,8 +2478,8 @@ _eval_plane(const fvm_selector_postfix_t  *pf,
       retval = true;
   }
   else {
-    assert(pf_type == PF_FLOAT);
     double epsilon = *((double *)(pf->elements + *i));
+    assert(pf_type == PF_FLOAT);
     *i += _postfix_float_size;
     if (FVM_ABS(pfunc) < epsilon)
       retval = true;
@@ -2529,18 +2529,20 @@ _eval_box(const fvm_selector_postfix_t  *pf,
 
   if (n_vals == 6) {
     if (   val[0] <= coords[0] && val[1] <= coords[1] && val[2] <= coords[2]
-        && val[3] >= coords[0] && val[4] >= coords[1] && val[5] >= coords[0])
+        && val[3] >= coords[0] && val[4] >= coords[1] && val[5] >= coords[2])
       retval = true;
   }
   else {
-    double _coords[3]
-      = {(coords[0] - val[0]), (coords[1] - val[1]), (coords[2] - val[2])};
-    double l12 = val[3]*val[3] + val[4]*val[4] + val[5]*val[5];
-    double l22 = val[6]*val[6] + val[7]*val[7] + val[8]*val[8];
-    double l32 = val[9]*val[9] + val[10]*val[10] + val[11]*val[11];
-    double dp1 = _coords[0]*val[3] + _coords[1]*val[4] + _coords[2]*val[5];
-    double dp2 = _coords[0]*val[6] + _coords[1]*val[7] + _coords[2]*val[8];
-    double dp3 = _coords[0]*val[9] + _coords[1]*val[10] + _coords[2]*val[11];
+    double _coords[3], l12, l22, l32, dp1, dp2, dp3;
+    _coords[0] = coords[0] - val[0];
+    _coords[1] = coords[1] - val[1];
+    _coords[2] = coords[2] - val[2];
+    l12 = val[3]*val[3] + val[4]*val[4] + val[5]*val[5];
+    l22 = val[6]*val[6] + val[7]*val[7] + val[8]*val[8];
+    l32 = val[9]*val[9] + val[10]*val[10] + val[11]*val[11];
+    dp1 = _coords[0]*val[3] + _coords[1]*val[4] + _coords[2]*val[5];
+    dp2 = _coords[0]*val[6] + _coords[1]*val[7] + _coords[2]*val[8];
+    dp3 = _coords[0]*val[9] + _coords[1]*val[10] + _coords[2]*val[11];
     if (   dp1 >= 0   && dp2 >= 0   && dp3 >= 0
         && dp1 <= l12 && dp2 <= l22 && dp3 <= l32)
       retval = true;
@@ -2588,20 +2590,22 @@ _eval_cylinder(const fvm_selector_postfix_t  *pf,
   /* Geometric test */
 
   {
-    double _coords[3]
-      = {(coords[0] - val[0]), (coords[1] - val[1]), (coords[2] - val[2])};
-    double _axis[3]
-      = {(val[3] - val[0]), (val[4] - val[1]), (val[5] - val[2])};
-    double dotp
-      = _coords[0]*_axis[0] + _coords[1]*_axis[1] + _coords[2]*_axis[2];
-    double len2 = _axis[0]*_axis[0] + _axis[1]*_axis[1] + _axis[2]*_axis[2];
+    double _coords[3], _axis[3], dotp, len2;
+    _coords[0] = coords[0] - val[0];
+    _coords[1] = coords[1] - val[1];
+    _coords[2] = coords[2] - val[2];
+    _axis[0] = val[3] - val[0];
+    _axis[1] = val[4] - val[1];
+    _axis[2] = val[5] - val[2];
+    dotp = _coords[0]*_axis[0] + _coords[1]*_axis[1] + _coords[2]*_axis[2];
+    len2 = _axis[0]*_axis[0] + _axis[1]*_axis[1] + _axis[2]*_axis[2];
     if (dotp >= 0 && dotp <= len2) {
+      double _proj[3], r2;
       double mult = dotp / len2;
-      double _proj[3]
-        = {(_coords[0] - mult*_axis[0]),
-           (_coords[1] - mult*_axis[1]),
-           (_coords[2] - mult*_axis[2])};
-      double r2 = _proj[0]*_proj[0] + _proj[1]*_proj[1] + _proj[2]*_proj[2];
+      _proj[0] = _coords[0] - mult*_axis[0];
+      _proj[1] = _coords[1] - mult*_axis[1];
+      _proj[2] = _coords[2] - mult*_axis[2];
+      r2 = _proj[0]*_proj[0] + _proj[1]*_proj[1] + _proj[2]*_proj[2];
       if (r2 <= val[6]*val[6])
         retval = true;
     }
@@ -2649,9 +2653,11 @@ _eval_sphere(const fvm_selector_postfix_t  *pf,
   /* Geometric test */
 
   {
-    double _coords[3]
-      = {(coords[0] - val[0]), (coords[1] - val[1]), (coords[2] - val[2])};
-    double len2
+    double _coords[3], len2;
+    _coords[0] = coords[0] - val[0];
+    _coords[1] = coords[1] - val[1];
+    _coords[2] = coords[2] - val[2];
+    len2
       = _coords[0]*_coords[0] + _coords[1]*_coords[1] + _coords[2]*_coords[2];
     if (len2 <= val[3]*val[3])
       retval = true;
